! VNY 25.01.2002; The subroutine dlocate can crash if the
! input xx-array consists of only one element. 
! Protection against this crash has been added

module numerical_mod

   use types
   use errorhandling 
   use sort_mod
   implicit none

   public

contains

subroutine polyfit(x,y,sigmay,npts,nterms,mode,a,chisqr)

   real(kind=double) x(:)
   real(kind=double) y(:)
   real(kind=double) sigmay(:)
   integer(kind=int32) npts
   integer(kind=int32) nterms
   integer(kind=int32) mode
   real(kind=double) a(:)
   real(kind=double) chisqr

   integer(kind=int32) nmax
   integer(kind=int32) n
   integer(kind=int32) j
   integer(kind=int32) i
   integer(kind=int32) k
   integer(kind=int32) l
   real(kind=double) chisq
   real(kind=double) weight
   real(kind=double) xterm
   real(kind=double) yterm
   real(kind=double) delta
   real(kind=double) free
   real(kind=double) xi
   real(kind=double) yi
   real(kind=double), allocatable :: array(:,:) 
   real(kind=double), allocatable :: sumx(:)
   real(kind=double), allocatable :: sumy(:)

   allocate(array(nterms, nterms))
   allocate(sumx(npts))
   allocate(sumy(npts))

   nmax  =  2 * nterms - 1
   sumx  =  0.0d0
   sumy  =  0.0d0
   chisq  =  0.0d0

   do i  =  1, npts
      xi  =  x(i)
      yi  =  y(i)
      if (mode < 0) then
         if (yi < 0) then
            weight  =  -1.0d0 / yi
         else if (yi == 0) then
            weight = 1.d0
         else
            weight = 1.0d0 / yi
         end if
      else if (mode == 0) then
         weight = 1.d0
      else
         weight = 1.0d0 / sigmay(i)**2
      end if

      xterm = weight
      do n = 1, nmax
         sumx(n) = sumx(n) + xterm
         xterm = xterm * xi
      end do

      yterm = weight * yi
      do n  =  1, nterms
         sumy(n) = sumy(n) + yterm
         yterm = yterm * xi
      end do

      chisq = chisq + weight * yi**2
   end do

   do j = 1, nterms
      do k = 1, nterms
         n = j + k - 1
         array(j, k) = sumx(n)
      end do
   end do







   delta = determ(array, nterms)
   if (delta /= 0) then
      do l = 1, nterms
         do j = 1, nterms
            do k = 1, nterms
               n  = j + k - 1
               array(j, k) = sumx(n)
            end do

            array(j, l) = sumy(j)
         end do

         a(l) = determ(array, nterms) / delta
      end do

      do j = 1, nterms
         chisq = chisq - 2.0d0 * a(j) * sumy(j)
         do k = 1, nterms
            n = j + k - 1
            chisq = chisq + a(j) * a(k) * sumx(n)
         end do
      end do

      free = dble(npts - nterms)
      chisqr = chisq / free
   else
      chisqr = 0.0d0
      do j = 1, nterms
         a(j) = 0.0d0
      end do
   end if

   deallocate(array)
   deallocate(sumx)
   deallocate(sumy)

end subroutine polyfit

function determ(array, norder) result(res)

   integer(kind=int32) :: norder
   real(kind=double) array(norder, norder)

   real(kind=double) res

   integer(kind=int32) i
   integer(kind=int32) j
   integer(kind=int32) k
   integer(kind=int32) k1
   real(kind=double) save

   res = 1.d0

   do k = 1, norder
      if (array(k, k) /= 0) goto 100

      do j = k, norder
         if (array(k,j) /= 0) goto 50
      end do

      res = 0.00
      goto 200

50    continue

      do i = k, norder
         save = array(i, j)
         array(i, j) = array(i, k)
         array(i, k) = save
      end do

      res = -res

100   continue

      res = res * array(k, k)

      if (k < norder) then
         k1 = k + 1

         do i = k1, norder
            do j = k1, norder
               array(i, j) = array(i, j) - &
                             array(i, k) * &
			     array(k, j) / &
			     array(k, k)
            end do
         end do
      end if
   end do

200 continue

end function determ
  
subroutine locate(xx, n, x, j)

   real xx(:)
   integer(kind=int32) n
   real x
   integer(kind=int32) j

   integer(kind=int32) ju
   integer(kind=int32) jl
   integer(kind=int32) jm
   
   j = 0

   if (x < xx(1)) then
      j = 1
   endif

   if (x > xx(n)) then
      j = n
   endif


   if ( j == 0) then

      if (x .eq. xx(n)) then
         ju = n-1
         jl = n-1
      endif

      if (x .eq. xx(1)) then
         jl = 1
         ju = 1 
      endif

      jl = 0
      ju = n + 1

      do while(ju - jl > 1)
         jm = (ju + jl) / 2
 
         if ((xx(n) > xx(1)) .eqv. (x > xx(jm))) then
            jl = jm
         else
            ju = jm
         end if
      end do

      j = jl
   endif

end subroutine locate



!*************************************************************
! subroutine dlocate
! VNY 25.01.2002; If the dimension of the array xx is 1 or 0
! this subroutin can crash due to subscript. Protection 
! against this crash has been added
! Modified 12.4.2002 to use arrays starting at subscript 0
!**************************************************************
subroutine dlocate(xx, n, x, j)
   integer (kind=int32), intent(in) :: n
   real(kind=double)   :: xx(0 : n - 1)
   
   real(kind=double)   :: x
   integer(kind=int32) :: j

   integer(kind=int32) :: ju
   integer(kind=int32) :: jl
   integer(kind=int32) :: jm

   jl = 0
   ju = n + 1

! VNY 25.01.2002 Protection against one-element array
   if (n .le. 1) then
      j = 0
      return
   endif

   if (x .ge. xx(n - 1)) then
       j = n - 1
       return
   end if 
      

   if (x .le. xx(0)) then
      j = 0
      return
   endif

   do while(ju - jl > 1)
      jm = (ju + jl) / 2

      if (x >= xx(jm - 1)) then
         jl = jm
      else
         ju = jm
      end if
   end do

   j = jl - 1

end subroutine dlocate

subroutine Marquardt(x, y, sig, nPts, param, ma, fitList, mFit, CovMatrix, &
                    alphaMatrix, nca, chiSqr, funcs, aLambda)

   integer(kind=int32) :: nPts
   real                :: x(nPts)
   real                :: y(nPts)
   real                :: sig(nPts)
   integer(kind=int32) :: ma
   real                :: param(ma)
   integer(kind=int32) :: fitList(*)
   integer(kind=int32) :: mFit
   integer(kind=int32) :: nca
   real                :: CovMatrix(nca, nca)
   real                :: chiSqr
   real                :: aLambda
   real                :: alphaMatrix(nca, nca)
   integer(kind=int32), parameter :: mmax = 20
   real                :: atry(mmax)
   real                :: beta(mmax)
   real                :: da(mmax)
   
   integer(kind=int32) :: j
   integer(kind=int32) :: k
   real ochiSqr

   external funcs

   if (aLambda < 0.0) then
!     VNY 29.01.03 corresction of possible errors
!      kk = mFit + 1

!      do j = 1, ma
!         ihit = 0

!         do k = 1, mFit
!            if (fitList(k) == j) ihit = ihit + 1
!         end do

!         if (ihit == 0) then
!            fitList(kk) = j
!            kk = kk + 1
!         else if (ihit > 1) then
!            call fatal("fitListWrong", "bad fitList")
!         end if
!      end do

!      if (kk /= (ma + 1)) then
!         call fatal("fitListWrong", "bad fitList")
!      end if
!--- Test code VNY 29.01.2003
      mFit=0
      do j=1,ma
         if (fitList(j) .eq. 1) mFit=mFit+1
      enddo
!---End test code

      aLambda = 0.001
      call MrqCof(x, y, sig, nPts, param, ma, fitList, mFit, alphaMatrix, beta, nca, &
                  chiSqr, funcs)
      ochiSqr = chiSqr
      atry(1:ma) = param(1:ma)
   end if

   do j = 1, mFit
      do k = 1, mFit
         CovMatrix(j, k) = alphaMatrix(j, k)
      end do

      CovMatrix(j, j) = alphaMatrix(j, j) * (1.0 + aLambda)
      da(j) = beta(j)
   end do

   call gaussj(CovMatrix, mFit, nca, da, 1, 1)

   if (aLambda == 0.0) then
      call CovarianceSort(CovMatrix, nca, ma, fitList, mFit)
      return
   end if

   do j = 1, mFit
      atry(fitList(j)) = param(fitList(j)) + da(j)
   end do

   call MrqCof(x, y, sig, nPts, atry, ma, fitList, mFit, CovMatrix, da, nca, &
               chiSqr, funcs)

   if (chiSqr < ochiSqr) then
      aLambda = 0.1 * aLambda
      ochiSqr = chiSqr

      do j = 1, mFit
         do k = 1, mFit
            alphaMatrix(j, k) = CovMatrix(j,k)
         end do

         beta(j) = da(j)
         param(fitList(j)) = atry(fitList(j))
      end do

   else
      aLambda = 10.0 * aLambda
      chiSqr = ochiSqr
   end if

end subroutine Marquardt

subroutine MrqCof(x, y, sig, nPts, param, ma, fitList, mFit, alphaMatrix, beta, nalp, &
                 chiSqr, funcs)

   integer(kind=int32) nPts
   real x(nPts)
   real y(nPts)
   real sig(nPts)
   integer(kind=int32) ma
   real param(ma)
   integer(kind=int32) fitList(*)
   integer(kind=int32) mFit
   integer(kind=int32) nalp
   real alphaMatrix(nalp, nalp)
   real beta(ma)
   real chiSqr
   real sig2i,dy,ymod,wt

   integer(kind=int32) i
   integer(kind=int32) j
   integer(kind=int32) k

   external funcs

   integer(kind=int32), parameter :: mmax = 20
   real dyda(mmax)

   do j = 1, mFit
      do k = 1, j
         alphaMatrix(j, k) = 0.0
      end do

      beta(j) = 0.0
   end do

   chiSqr = 0.0

   do i = 1, nPts
      call funcs(x(i), param, ymod, dyda, ma)

      sig2i = 1.0 / (sig(i) * sig(i))
      dy = y(i) - ymod

      do j = 1, mFit
         wt = dyda(fitList(j)) * sig2i

         do k = 1, j
            alphaMatrix(j, k) = alphaMatrix(j, k) + wt * dyda(fitList(k))
         end do

         beta(j) = beta(j) + dy * wt
      end do

      chiSqr = chiSqr + dy * dy * sig2i
   end do

   do j = 2, mFit
      do k = 1, j - 1
         alphaMatrix(k, j) = alphaMatrix(j, k)
      end do
   end do

   return

end subroutine MrqCof

subroutine GaussJ(param, n, np, b, m, mp)

   integer(kind=int32) np
   real param(np, np)
   integer(kind=int32) n
   integer(kind=int32) mp
   real b(np, mp)
   integer(kind=int32) m

   integer i,j,k
   integer, parameter :: nmax=50
   real big,pivinv,dum
   integer irow, icol, l, ll
   integer ipiv(nmax),indxr(nmax),indxc(nmax)

   ipiv(1:n) = 0

   do i = 1, n
      big = 0.0

      do j = 1, n
         if (ipiv(j) /= 1) then
            do k = 1, n
               if (ipiv(k) == 0) then
                  if (abs(param(j, k)) >= big) then
                     big = abs(param(j, k))
                     irow = j
                     icol = k
                  end if
               else if (ipiv(k) > 1) then
                  call fatal("sigMat", "Singular matrix in Marquardt minimization")
               end if
            end do
         end if
      end do

      ipiv(icol) = ipiv(icol) + 1

      if (irow /= icol) then
         do l = 1, n
            dum = param(irow, l)
            param(irow, l) = param(icol, l)
            param(icol, l) = dum
         end do

         do l = 1, m
            dum = b(irow, l)
            b(irow, l) = b(icol, l)
            b(icol, l) = dum
         end do
      end if

      indxr(i) = irow
      indxc(i) = icol

      if (param(icol, icol) == 0.0) then
         call fatal("sigMat", "Singular matrix in Marquardt minimization")
      end if

      pivinv = 1.0 / param(icol, icol)
      param(icol, icol) = 1.0

      do l = 1, n
         param(icol, l) = param(icol, l) * pivinv
      end do

      do l = 1, m
         b(icol, l) = b(icol, l) * pivinv
      end do

      do ll = 1, n
         if (ll /= icol) then
            dum = param(ll, icol)
            param(ll, icol) = 0.0

            do l = 1, n
               param(ll, l) = param(ll, l) - param(icol, l) * dum
            end do

            do l = 1, m
               b(ll, l) = b(ll, l) - b(icol, l) * dum
            end do
         end if
      end do
   end do

   do  l = n, 1, -1
      if (indxr(l) /= indxc(l)) then
         do k = 1, n
            dum = param(k, indxr(l))
            param(k, indxr(l)) = param(k, indxc(l))
            param(k, indxc(l)) = dum
         end do
      end if
   end do

end subroutine GaussJ

subroutine CovarianceSort(CovMatrix, nCov, ma, fitList ,mFit)
  integer :: nCov, mFit, i, j, ma
  real :: CovMatrix(nCov,nCov)
  integer :: fitList(*)
  real :: swapval
  do  j = 1, ma - 1
     do i = j+1, ma
        CovMatrix(i,j) = 0.
     enddo
  enddo

  do  i = 1, mFit-1
     do  j = i+1, mFit
        if(fitList(j) > fitList(i)) then
           CovMatrix(fitList(j),fitList(i))=CovMatrix(i,j)
        else
           CovMatrix(fitList(i),fitList(j))=CovMatrix(i,j)
        endif
     enddo
  enddo

  swapval = CovMatrix(1,1)

  do  j = 1, ma
     CovMatrix(1,j) = CovMatrix(j,j)
     CovMatrix(j,j) = 0.
  enddo
  CovMatrix(fitList(1),fitList(1)) = swapval
  do j = 2, mFit
     CovMatrix(fitList(j), fitList(j)) = CovMatrix(1,j)
  enddo
  do  j = 2, ma
     do i = 1, j-1
        CovMatrix(i,j)=CovMatrix(j,i)
     enddo
  enddo
end subroutine CovarianceSort

real function computeMedian(image, nx, ny, i1, i2, j1, j2) result (median)

   ! computes the median of an image over the specific limits

   integer :: nx, ny
   real :: image(0:nx-1, 0:ny-1)
   integer :: i1, i2, j1, j2
   integer :: i, j
   real, allocatable :: array(:)
   integer :: npts
   
   npts = (i2-i1+1)*(j2-j1+1)
   allocate(array(npts))

   npts = 0
   do i = i1, i2
      do j = j1, j2
         npts = npts + 1
         array(npts) = image(i,j)
      enddo
   enddo

   call sort(npts, array)

   if (NINT(real(npts)/2.) /= INT(npts/2)) then
      median = array(NINT(real(npts)/2.))
       else
      median = 0.5*(array(npts/2) + array(npts/2+1))
   endif

end function computeMedian


real function arrayMedian(array, npts) result (median)

   ! computes the median of an array

   integer :: npts
   real :: array(*)
   

   if (npts .eq. 1) then
      median = array(1)
    else
      call sort(npts, array)
      if (NINT(real(npts)/2.) /= INT(npts/2)) then
        median = array(NINT(real(npts)/2.))
         else
        median = 0.5*(array(npts/2) + array(npts/2+1))
      endif
   endif

!   write (*, *) "npts : ", npts
!   do i = 1, npts
!      write (*, *) i, array(i)
!   end do

end function arrayMedian

end module numerical_mod



