module omcelestial_mod

  use oal
  use cal
  use dal
  use sasdal
  use caloaldefs

  use errorHandling
  use sla

 
  use omconstants_mod
  use omdistortion_mod
contains


!*******************************************************************
!
!
!******************************************************************
subroutine ComputeCelestialCoordinates(xPos, yPos, x0, y0, &
                                       xBinning, yBinning, &
                                       plateScale, rollAngle, &  
                                       ra_point, dec_point, &
                                       rightAscension, declination , &
                                       galacticLong, galacticLat)
    real(single), intent(in) :: xPos, yPos
    real(single), intent(in) :: x0, y0
 
    real(single), intent(in) :: xBinning, yBinning
    real(double), intent(in) :: plateScale, rollAngle
    real(double), intent(in) :: ra_point, dec_point
  
    real(double), intent(out) :: rightAscension, declination
    real(double), intent(out) :: galacticLong, galacticLat
    real(double) ::  xposcorr,  yposcorr
    real(double) :: xi_source, eta_source
    real(double) :: xrad, yrad              ! position in radians     
    real(double) :: cosRollAngle, sinRollAngle
    integer(int32) :: mode

    cosRollAngle = dcos(rollAngle)
    sinRollAngle = dsin(rollAngle)
      

    xposcorr = dble(xpos) * dble(xBinning) + dble(x0)  
    yposcorr = dble(ypos) * dble(ybinning) + dble(y0)
 
    !***************************************************
    ! correct positions for pincushion/barrel distortion
    !***************************************************
  
    rawx(0) = real(xposcorr)
    rawy(0) = real(yposcorr)


    mode = 1
    call CAL_omDistortion(rawx, rawy, deltax, deltay, sigmax, sigmay, mode)


    xposcorr = xposcorr - dble(deltax(0))  ! - sign removes distortion    
    yposcorr = yposcorr - dble(deltay(0))  ! - sign removes distortion

           
    !***************************************************
    ! get offset relative to tangent point (at OM centre)
    !***************************************************
    xrad = xposcorr - xCentre 
    yrad = yposcorr - yCentre


    !**************************************************
    !  rotate x,y frame to tangent sky (RA, DEC) frame
    !  and form tangent-plane coords
    !*************************************************
    xi_source = -plateScale * (xrad * cosRollAngle - yrad * sinRollAngle)
    eta_source = plateScale * (xrad * sinRollAngle + yrad * cosRollAngle)


    !***********************************************************
    ! Transform tangent-plane coordinates (xi_source, yi_source) 
    ! into spherical (sky) coordinates (RA, dec). ra_point,
    ! dec_point is the spherical coordinate of the tangent-point
    ! Also calculate the galactic longitude and latitude of
    ! each source
    !************************************************************
     
    call sla_dtp2s(xi_source, eta_source, &
            ra_point, dec_point, rightAscension, declination)

    call sla_EQGAL(rightAscension,  declination, &
                      galacticLong, galacticLat)


end subroutine ComputeCelestialCoordinates


end module omcelestial_mod



