!
module ommagnitude_mod

   use param
   use cal
   use dal
   use errorHandling
   use frameTime_mod
   use omallocation_mod
   use omutility_mod
   use getFilterId_mod
   implicit none

   private

   public ComputeMagnitude
   public TDPCorrectionFactor

!   public PSFCountsRatio
contains


!*************************************************************
!
! ComputeMagnitude
!
!*************************************************************
subroutine ComputeMagnitude(sourceRate, sourceRateErr, &
                            backgroundRate, backgroundRateErr, &
                            xCoord, yCoord,             &
                            scaleCounts, extended, &
                            photometryApertureRadius, calApertureRadius, &
                            frameTime, deadFrac,                        &
                            magnitude, magnitudeError, psf2Correction, &
                            filter, mjd, tdpCorrFactor)


    real(single), intent(inout) :: sourceRate, sourceRateErr, backgroundRate, backgroundRateErr
    real(single), intent(in) :: xCoord, yCoord
    real(single), intent(in) ::  photometryApertureRadius, calApertureRadius
    real(double), intent(in) :: frameTime, deadFrac
    character(*), intent(in), optional :: filter
    real(double), intent(in), optional :: mjd 
    real(single), intent(out), optional :: tdpCorrFactor
    real(single), intent(out) :: magnitude, magnitudeError, psf2Correction 

    logical, intent(in) :: scaleCounts
    integer(int8), intent(in) :: extended
    real(double) :: corrMag                                      ! Corrected magnitude.
    real(double) :: corrMagErrPlus                               ! Corrected magnitude error + (asymmetric).
    real(double) :: corrMagErrMinus                              ! Corrected magnitude error - (asymmetric).
    real(double) :: countsRatio = 1.0_double
    real(double) :: scaleCountsRatio = 1.0_double
  
    real(single) :: countFrameRatio = 0.5
    real(double) :: deadFractionCorrection
    real(single) :: tdpCorrFactor1
 
    deadFractionCorrection = 1.0_double / (1.0_double - deadFrac)

    if (backgroundrateerr .gt. 10.0) then

    end if


!write(*, *) "mssllib ommagnitude_mod line 67 frametime=", frameTime, " deadfrac=", deadfrac
    
    !******************************************************
    !
    ! For UV data need to scale the counts from 12 to 35 pixels
    ! Using psf function
    !
    !******************************************************
    if (sourceRate .gt. 0.0) then 
               
!write(*, *) "ommagniutde x=", xcoord, " y=", ycoord
!write(*, *) "ommagnitude line 60- scale counts=", scaleCounts, " extended=", extended
!write(*, *) "photometryApertureRadius=", photometryApertureRadius, " calApertureRadius=",  calApertureRadius                     
!write(*, *) "frametime=", frameTime, " deadfrac=", deadFrac
!write(*, *) "source rate=", sourceRate, " error=", sourceRateErr
!write(*, *) "background ate=", backgroundRate, " error=", backgroundRateErr

                
        !***************************************************************
        !
        ! If necessary, scale the counts to the CAL photometric radius
        !
        !***************************************************************
        if (scaleCounts .and. extended .eq. 0) then
            if (frameTime < 1e-6) then
                countFrameRatio = 0.1
            else        
                countFrameRatio = min(real(sourceRate * frameTime), 1.0) 

                !*************************************
                ! Ensure ratio is between 0 and 1
                !*************************************     
                countFrameRatio = max(0.0, countFrameRatio)
            end if     





            countsRatio = PSFCountsRatio(photometryApertureRadius, calApertureRadius, &
                                     xCoord, yCoord, countFrameRatio)
                
 
            sourceRate = sourceRate * countsRatio
            sourceRateErr = sourceRateErr * countsRatio
            psf2Correction = (countsRatio - 1.0) * 100.0

            scaleCountsRatio = photometryApertureRadius * photometryApertureRadius / &
                 (calApertureradius * calApertureRadius)



            backgroundRate = backgroundrate * scaleCountsRatio 
        else
            psf2Correction = 0.0           
 
        end if

!write(*, *) "Line 121 countsratio=", countsRatio, " scalecountsratio=", scaleCountsRatio
!write(*, *) "dead fraction correction=", deadFractionCorrection

        !************************************************
        !
        ! Correct for dead-fraction
        !
        !************************************************
        sourceRate = sourceRate * deadFractionCorrection
        sourceRateErr = sourceRateErr * deadFractionCorrection            
        backgroundRate = backgroundRate * deadFractionCorrection

!write(*, *) "Corrected source rate=", sourceRate   

        !************************************************************
        !
        ! Calculate the instrumental magnitude
        !
        !***********************************************************
        if (present(mjd) .and. present(filter)) then

!write(*, *) "Line 137 filter=", filter, " mjd=", mjd

            !**************************************************************
            ! Correction for time-dependence-sensistivity
            !**************************************************************
            tdpCorrFactor1 = TDPCorrectionFactor(mjd, filter)
            sourceRate = tdpCorrfactor1 * sourceRate
            sourceRateErr = tdpCorrfactor1 * sourceRateErr

            backgroundRate = tdpCorrfactor1 * backgroundRate
            if (present(tdpcorrfactor)) tdpcorrfactor = tdpcorrfactor1

        end if



        call CAL_omPhotoMagnitude(dble(sourceRate), corrMag, -dble(sourceRateErr), &
                                  dble(sourceRateErr), corrMagErrMinus, corrMagErrPlus)


!write(*, *) "tdp correction=", tdpCorrfactor1
!write(*, *) "corrected source rate=", sourceRate, " background rate=", backgroundrate
!write(*, *) "magnitude=", corrMag, " deadfractioncorrection=", deadfractioncorrection




        magnitude = real(corrMag)
!        magnitudeError = real(abs(max(corrMagErrMinus, corrMagErrPlus)))
   
        magnitudeError = 0.4342945 * 2.5 * sourceRateErr / sourceRate
      


    end if

!write(*, *) "magnitude=", magnitude, " error=", magnitudeError

!write(*, *) "Psf2 correction=", psf2correction, " deadfractioncorrection=",  deadFractionCorrection 





end subroutine ComputeMagnitude 
!**********************************************************************************
!
! Returns the time-dependent sensitivity correction factor
!
! The correction factor is Y = a + b * mjd, where mjd is the modified Julian date
! The corrected count-rates are given by corrected rate = measured rate * Y
!**********************************************************************************

real(single) function TDPCorrectionFactor(mjd, filter)
    real(double), intent(in) :: mjd  ! Modified Julian day
    character(*), intent(in) :: filter
    real(double) :: sum, arg
    integer :: i


    real(double), pointer :: coeffs(:)=> null()



!    select case (trim(filter))
!        case ("U")
!            coeffs = (/-0.313603, 2.51781e-05/)
!        case ("B")
!            coeffs = (/-0.484320, 2.84507e-05/)
!        case ("V")
!            coeffs = (/-1.25429, 4.32118e-05/)
!        case ("UVW1")
!            coeffs = (/-1.16448, 4.14899e-05/)
!        case ("UVW2")
!            coeffs = (/-2.72723, 7.14543e-05/)
!        case ("UVM2")
!            coeffs = (/-2.91093, 7.49769e-05/)
!        case default
!            coeffs = (/1.0, 0.0/)
!     end select 



    select case (trim(filter))
        case ("U", "B", "V", "UVW1", "UVW2", "UVM2", &
              "u", "b", "v", "uvw1", "uvw2", "uvm2")
  
            call CAL_setState(OM, filterId =  filterIdToInt(trim(filter)))

            call CAL_omGetDegradationCoeffs( coeffs )

            sum = coeffs(0)
            arg = mjd

            do i = 1, size(coeffs) - 1
                sum = sum + arg * coeffs(i)  
                arg = arg * mjd  

            end do
            TDPCorrectionFactor = sum 
        case default

            TDPCorrectionFactor = 1.0
       
    end select


!    TDPCorrectionFactor = coeffs(0) + coeffs(1) * mjd
    



    
end function TDPCorrectionFactor



!************************************************************
! Function PSFCountsRatio
! Computes the ratio of the total counts between
! the radii radius1 and radius2 
!************************************************************
real(double) Function PSFCountsRatio(radius1, radius2, x0, y0, countFrameRatio)
    real(single), intent(in) :: radius1    ! The aperture radius that the input counts refer to
    real(single), intent(in) :: radius2    ! The radius that the counts have to be scaled to (NB WRONG WAY ROUND!)
    real(single), intent(in) :: x0         ! The x coordinate of the source 
    real(single), intent(in) :: y0         ! The y coordinate of the source 

    real(single), intent(in) :: countFrameRatio

    PSFCountsRatio = PSFFractionOfTotalCounts(x0, y0, radius1, countFrameRatio) / &
                  PSFFractionOfTotalCounts(x0, y0, radius2, countFrameRatio)


end function PSFCountsRatio  


!******************************************
! PSFFractionOfTotalCounts
! Returns the fraction of the total counts
! contained within the given radius
!******************************************
real(double) function & 
    PSFFractionOfTotalCounts(x0, y0, radius, countFrameRatio)
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: radius
    real(single), intent(in) :: countFrameRatio
    type(OmPsfT) :: psf
 
    psf = CAL_omGetPSF(dble(x0), dble(y0), dble(countFrameRatio))

    PSFFractionOfTotalCounts = CAL_omPsfEncircledEnergy(psf, dble(radius))


end function PSFFractionOfTotalCounts 



end module ommagnitude_mod












