module ompsf_mod
use dal
use omallocation_mod
use splinefit_mod
use cubicshepard_mod
use ominterpolate_mod

use gridinterpolation_mod
use ompsfintegration_mod
use omutility_mod

implicit none




interface CreatePSF
module procedure  CreatePSF1,  CreatePSF2 , &
                  CreatePSF3,  CreatePSF4, &
                  CreatePSF5,  CreatePSF6, &
                  CreatePsfimage21 


end interface  CreatePSF


contains


subroutine CreatePSF1(xPos, yPos, ratio, binX, binY, &
                  xres, yres, totalCounts, spline)
    real(double), intent(in) :: xPos, yPos, ratio
 
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    type(TwoDSpline), intent(out) :: spline    
    integer(int32) :: nx, ny
    nx = 25
    ny = 25

    if (present(totalCounts)) then
        call CreatePSF2(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, spline)
    else
        call CreatePSF2(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, psfSpline = spline)
    end if    
!write(*, *) "Line 40 leaving createpsf1"
end subroutine CreatePSF1

!**************************************************************************
!
!
!**************************************************************************
subroutine CreatePSF2(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, psfSpline)
    real(double), intent(in) :: xPos, yPos, ratio
    integer (int32), intent(inout) :: nx, ny
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    real(double) :: x1, y1    
    type(TwoDSpline), intent(out) :: psfSpline    
  

    real (single), pointer :: psfImage(:, :)=>Null()  ! Pointer to the psf image
    real(single), pointer :: x(:) => null(), y(:) => null() 

    real(double) :: xCentre, yCentre, f, fCentre       
    integer :: i, j
    
  
    
    !******************************************************
    ! Create the PSF  -image map
    !*****************************************************


    psfImage => CreatePSFImage(nx, ny, xPos, yPos, ratio, xRes, yRes)  ! 5 is the resolution


    if (present(totalCounts)) then
        if (totalCounts .gt. 0.0) then
            f = sum(psfImage)

            psfImage = psfImage * totalCounts  / f

!write(*, *) "Total counts of psf image=", sum(psfImage), " requied counts=", totalCounts

        else
            f = maxval(psfImage)
        
            psfImage = psfImage * abs(totalCounts) / f
            totalCounts = sum(psfImage)
!write(*, *) "Line 88 Total counts of psf image=", sum(psfImage), " input counts=", totalCounts

        end if
    else
        f = maxval(psfImage)
        
        psfImage = psfImage / f
    end if



    nx = size(psfImage, 1)
    ny = size(psfImage, 2)




 ! Noraml image so that total counts = totalCounts
  



    x => allocateSingle(1, nx, " x")
    y => allocateSingle(1, ny, " y")
    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0


    do i = 1, nx
        x(i) = (real(i) - 0.5_double  - xCentre) / real(xRes)
    end do
    do j = 1, ny
        y(j) = (real(j) - 0.5_double  - yCentre)  / real(yRes)
    end do



    



    call ConstructSpline(nx, ny, x, y, psfImage, psfSpline)

    fCentre = interpolateSpline(0.0_double, 0.0_double, psfSpline)



!write(*, *) "Interpolated spline counts at centre=", fCentre


    if (binX .ne. 1) nx = nx / 2
    if (binY .ne. 1) ny = ny / 2



    deallocate(psfImage)
    psfImage => allocate2dSingle(0, nx - 1, 0, ny - 1, "psfimage")
    psfImage = 0.0

!write(*, *) "Line 138"

    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0




    do j = 1, ny
        y1 = (dble(j) - 0.5_double  - yCentre) * real(binY)
      
  
        do i = 1, nx
            x1 = (dble(i) - 0.5_double  - xCentre) * real(binx)
           
            f= interpolateSpline(x1, y1, psfSpline)
!            if (abs(f - psfImage(i - 1, j - 1)) > 1e-3) then 
!                write(*, *) "f=", f, " actual image value=", psfImage(i - 1, j - 1) 
!                call fatal("bug", "spline interpolation problem ")
!            end if
            psfImage(i - 1, j - 1) = f
        end do
    end do
!write(*, *) "Line 163"
    call DestructSpline(psfSpline)
    call ConstructSpline(nx, ny, x, y, psfImage, psfSpline)


    deallocate(x, y)
    nullify(x, y)



!call SaveImage(psfImage, nx, ny, "psfimage3.fit", "test")
   
    deallocate(psfImage)
    nullify(psfImage)
!write(*, *) "Line 177"

end subroutine CreatePSF2

!******************************************************************
!
!*****************************************************************
subroutine CreatePSF3(xPos, yPos, ratio, binX, binY, &
                  xres, yres, totalCounts, shepard)

    real(double), intent(in) :: xPos, yPos, ratio
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    type(structShepard), intent(out) :: shepard    
    integer(int32) :: nx, ny

    nx = 25
    ny = 25

    if (present(totalCounts)) then
        call CreatePSF4(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, shepard)
    else
        call CreatePSF4(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, psfShepard = shepard)
    end if    

end subroutine CreatePSF3

!**************************************************************************
!
!
!**************************************************************************
subroutine CreatePSF4(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, psfShepard)
    real(double), intent(in) :: xPos, yPos, ratio
    integer (int32), intent(inout) :: nx, ny
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    real(double) :: x1, y1    
    type(structShepard), intent(out) :: psfShepard
  

    real (single), pointer :: psfImage(:, :)=>Null()  ! Pointer to the psf image
    real(single), pointer :: x(:) => null(), y(:) => null() 

    real(double) :: xCentre, yCentre, f, fCentre       
    integer :: i, j, m
         
    !******************************************************
    ! Create the PSF  -image map
    !*****************************************************

    psfImage => CreatePSFImage(nx, ny, xPos, yPos, ratio, xRes, yRes)  ! 5 is the resolution


    if (present(totalCounts)) then
        if (totalCounts .gt. 0.0) then
            f = sum(psfImage)

            psfImage = psfImage * totalCounts  / f
        else
            f = maxval(psfImage)
        
            psfImage = psfImage / f
            totalCounts = sum(psfImage)
        end if
    else
        f = maxval(psfImage)
        
        psfImage = psfImage / f
    end if



    nx = size(psfImage, 1)
    ny = size(psfImage, 2)




 ! Noraml image so that total counts = totalCounts
  

!write(*, *) "Total counts of psf image=", sum(psfImage), " requied counts=", totalCounts

    m = nx * ny
    x => allocateSingle(1, m, " x")
    y => allocateSingle(1, m, " y")
    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0


    do i = 1, nx
        x(i) = (real(i) - 0.5_double  - xCentre) / real(xRes)
    end do
    do j = 1, ny
        y(j) = (real(j) - 0.5_double  - yCentre)  / real(yRes)
    end do





!write(*, *) "Line 271"

    call ShepardConstruct(nx, ny, x, y, psfImage, psfShepard)
!write(*, *) "Line 274"


    fCentre = shepardInterpolate(psfShepard, 0.0_double, 0.0_double)

!write(*, *) "Shepard Interpolated spline counts at centre=", fCentre


    if (binX .ne. 1) nx = nx / 2
    if (binY .ne. 1) ny = ny / 2



    deallocate(psfImage)
    psfImage => allocate2dSingle(0, nx - 1, 0, ny - 1, "psfimage")
    psfImage = 0.0



    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0




    do j = 1, ny
        y1 = (dble(j) - 0.5_double  - yCentre) * real(binY)
      
  
        do i = 1, nx
            x1 = (dble(i) - 0.5_double  - xCentre) * real(binx)
           
            f= Shepardinterpolate(psfShepard, x1, y1)
            if (abs(f - psfImage(i - 1, j - 1)) > 1e-3) then 
!                write(*, *) "f=", f, " actual image value=", psfImage(i - 1, j - 1) 
!                call fatal("bug", "spline interpolation problem ")
            end if
            psfImage(i - 1, j - 1) = f
        end do
    end do
    psFImage = psFImage

    deallocate(x, y)
    nullify(x, y)

!call SaveImage(psfImage, nx, ny, "psfimage3.fit", "test")

   
    deallocate(psfImage)
    nullify(psfImage)


end subroutine CreatePSF4

!******************************************************************
!
!*****************************************************************

subroutine CreatePSF5(xPos, yPos, ratio, binX, binY, &
                  xres, yres, totalCounts, grid)
    real(double), intent(in) :: xPos, yPos, ratio
 
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    type(GridInterpolant), intent(out) :: grid    
    integer(int32) :: nx, ny

    nx = 25
    ny = 25

    if (present(totalCounts)) then
        call CreatePSF6(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, grid)
    else
        call CreatePSF6(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, psfGrid = grid)
    end if    

end subroutine CreatePSF5

!**************************************************************************
!
!
!**************************************************************************
subroutine CreatePSF6(xPos, yPos, ratio, nx, ny, binX, binY, xres, yres, totalCounts, psfGrid)
    real(double), intent(in) :: xPos, yPos, ratio
    integer (int32), intent(inout) :: nx, ny
    integer (int32), intent(in) :: binX, binY, xres, yres
    real(double), optional, intent(inout) :: totalCounts
    real(double) :: x1, y1    
    type(GridInterpolant), intent(out) :: psfGrid
  

    real (single), pointer :: psfImage(:, :)=>Null()  ! Pointer to the psf image
    real(single), pointer :: x(:) => null(), y(:) => null() 

    real(double) :: xCentre, yCentre, f, fCentre       
    integer :: i, j, m
         
    !******************************************************
    ! Create the PSF  -image map
    !*****************************************************

    psfImage => CreatePSFImage(nx, ny, xPos, yPos, ratio, xRes, yRes)  ! 5 is the resolution


    if (present(totalCounts)) then
        if (totalCounts .gt. 0.0) then
            f = sum(psfImage)

            psfImage = psfImage * totalCounts  / f
        else
            f = maxval(psfImage)
        
            psfImage = psfImage / f
            totalCounts = sum(psfImage)
        end if
    else
        f = maxval(psfImage)
        
        psfImage = psfImage / f
    end if



    nx = size(psfImage, 1)
    ny = size(psfImage, 2)




 ! Noraml image so that total counts = totalCounts
  

!write(*, *) "Total counts of psf image=", sum(psfImage), " requied counts=", totalCounts

    m = nx * ny
    x => allocateSingle(1, m, " x")
    y => allocateSingle(1, m, " y")
    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0


    do i = 1, nx
        x(i) = (real(i) - 0.5_double  - xCentre) / real(xRes)
    end do
    do j = 1, ny
        y(j) = (real(j) - 0.5_double  - yCentre)  / real(yRes)
    end do





!write(*, *) "Line 443- constructing grid interpolant nx=", nx, " ny=", ny

    call GridInterpolantConstruct(nx, ny, x, y, psfImage, psfGrid)
!write(*, *) "Line 274"


    fCentre = GridInterpolate(0.0_double, 0.0_double, psfGrid)

!write(*, *) "Shepard Interpolated spline counts at centre=", fCentre


    if (binX .ne. 1) nx = nx / 2
    if (binY .ne. 1) ny = ny / 2



    deallocate(psfImage)
    psfImage => allocate2dSingle(0, nx - 1, 0, ny - 1, "psfimage")
    psfImage = 0.0



    xCentre = real(nx) / 2.0
    yCentre = real(ny) / 2.0

    do j = 1, ny
        y1 = (dble(j) - 0.5_double  - yCentre) * real(binY)
      
        do i = 1, nx
            x1 = (dble(i) - 0.5_double  - xCentre) * real(binx)
           
            f= gridinterpolate(x1, y1, psfGrid)
            if (abs(f - psfImage(i - 1, j - 1)) > 1e-3) then 
!                write(*, *) "Gridinterpolant f=", f, " actual image value=", psfImage(i - 1, j - 1) 
!                call fatal("bug", "spline interpolation problem ")
            end if
            psfImage(i - 1, j - 1) = f
        end do
    end do




    psFImage = psFImage

    deallocate(x, y)
    nullify(x, y)

!call SaveImage(psfImage, nx, ny, "psfimage3.fit", "test")

   
    deallocate(psfImage)
    nullify(psfImage)


end subroutine CreatePSF6


!********************************************************
! Subroutine CreatePSFImage
! Purpose - Creates point-spread function image centred
! at x0, y0
!********************************************************
function CreatePSFImage(psfNx, psfNy, x0, y0, countFrameRatio, xRes1, yRes1) result(psfImage)
    integer(int32), intent(inout) :: psfNx, psfNy
    real(double), intent(in) :: x0, y0 
    real(double), intent(in) :: countFrameRatio
    integer(int32), optional, intent(in) :: xRes1, yRes1
    real(single), dimension(:, :), pointer :: psf ! To hold the point-spread function
    integer(int32) ::  xRes, yRes
    real(single), pointer :: psfImage(:, :)     
    real(double) :: f
    !********************************
    ! Get the point-spread function
    !********************************
!    psfNx = 12   ! was 10
!    psfNy = 12   ! was 10
   
    if (present(xRes1)) then
        xRes = xRes1
    else
        xres = 1
    end if

    if (present(yRes1)) then
        yRes = yRes1
    else
        yres = 1
    end if
          
    call CAL_omGetPSFmap(dble(x0), dble(y0), &
                psfNx, psfNx, psfNy, psfNy, &
                xRes, yRes, dble(countFrameRatio), psf)
 
    psfNx = size(psf, 1) 
    psfny = size(psf, 2)
    psfImage => Allocate2DSingle(0, psfNx - 1, 0,  psfNy - 1, "psfImage (CreatePSFImage)")

    
    f = maxval(psf)
    psfImage = psf / f
  
   
    call CAL_releaseMemory(psf)
    


end function CreatePSFImage



!**************************************************************************
!
!
!
!
!**************************************************************************
subroutine CreatePSFImage21(psfNx, psfNy, psfimageOut, xPos, yPos, &
                          ratio, bin, res,totalCounts,           &
                          psfGridFitOut, psfsplinefit, bilinearData)


    real (single), optional, pointer :: psfImageOut(:, :) ! Pointer to the psf image

    real(double), intent(in) :: xPos, yPos, ratio
    integer (int32), intent(inout) :: psfnx, psfny
    integer (int32), intent(in) :: bin
    integer, optional, intent(in) :: res
    real(single), intent(inout), optional :: totalCounts
    real(double) :: x1, y1    
    type(GridInterpolant), optional, intent(out) :: psfGridFitOut
   
    type(TwoDSpline), optional, intent(out) :: psfSplineFit    
    type(BilinearStruct), optional, intent(inout) :: bilinearData 

    real(single), pointer :: x(:) => null(), y(:) => null(), height(:, :) => null() 
    real (single), pointer :: psfImage(:, :) => null() ! Pointer to the psf image


    real(double) :: xCentre, yCentre, f, f1, f2, f3, f4, fCentre
    integer :: i, j, m
    integer :: xRes, yRes, nx, ny  
    type(GridInterpolant) :: psfGridFit1

    real(single), dimension(:, :), pointer :: psf => null() ! To hold the point-spread function

    type(SourceParameters) :: source
  
 
    !******************************************************
    ! Create the PSF  -image map
    !*****************************************************

    if (present(res)) then
        xres = res
        yres = res
    else
        xres = 1
        yres = 1
    end if


    !*******************************************
    !
    ! Get the point-spread function from cal
    !
    !*******************************************             
    call CAL_omGetPSFmap(xpos, ypos, &
                psfNx, psfNx, psfNy, psfNy,  &
                xres, yres, dble(ratio), psf)
 


  

    !*************************************
    ! Normalize so that central peak is 1
    !*************************************
    f = maxval(psf)    
    psf = psf / f
  
          
    psfnx = size(psf, 1)
    psfny = size(psf, 2)


    ! Construct a grid-interpolant

    m = psfnx * psfny
    x => allocateSingle(1, psfnx, " x")
    y => allocateSingle(1, psfny, " y")

    xCentre = dble(psfnx) / 2.0_double + 0.5_double
    yCentre = dble(psfny) / 2.0_double + 0.5_double


    source%xpos = xcentre ! NB centres are at the middle of the pixel, pixel(0, 0) = 1, 1
    source%ypos = ycentre



    do i = 1, psfnx
        x(i) = dble(i)
    end do

    do j = 1, psfny
        y(j) = dble(j) 
    end do


    call GridInterpolantConstruct(psfnx, psfny, x, y, psf, psfGridFit1)


    fCentre = GridInterpolate(xCentre, yCentre, psfGridFit1)


    if(xres .gt. 1) then
        nx = psfnx / xres
        ny = psfny / yres
     else
        nx = psfnx
        ny = psfny
    end if


    if (bin .eq. 1) then ! No binning


     
        if (present(totalCounts)) then
            if (totalCounts .gt. 0.0) then
                f = sum(psf)

                psf = psf * totalCounts  / f

            else
                f = maxval(psf)
        
                psf = psf * abs(totalCounts) / f
                totalCounts = sum(psf)

            end if
        end if


        if (present(psfimageout)) then
            psfImageOut => allocate2dSingle(0, psfnx - 1, 0, psfny - 1, "psfimageout")
        
            psfImageout = psf


        end if 


        if (present(psfSplineFit)) then    

            call ConstructSpline(psfnx, psfny, x, y, psf, psfSplineFit)

            fCentre = interpolateSpline(0.0_double, 0.0_double, psfSplineFit)
 !           write(*, *) "Interpolated spline counts at centre=", fCentre

        end if


!write(*, *) "ompsf line 704"


        if(present(psfGridFitout)) then

!write(*, *) "ompsf line 709, xcentre=", xcentre, " ycentre=", ycentre
!write(*, *) "psfnx=", psfnx, " psfny=", psfny


            call GridInterpolantDestruct(psfGridFit1)
            do i = 1, psfnx
                x(i) = real(i) - xCentre
            end do

            do j = 1, psfny

                y(j) = real(j) - yCentre
            end do


            call GridInterpolantConstruct(psfnx, psfny, x, y, psf, psfGridFitOut)


!write(*, *) "Line 755  CreatePSFImage21- gridfit - interpolated value at 0, 0=",  &
!   gridinterpolate(0.0_double, 0.0_double, psfGridFitout)
!write(*, *) "Line 755 interpolated value at -1, -1=",  gridinterpolate(-1.0_double, -1.0_double, psfGridFitout)
!write(*, *) "Line 755 interpolated value at 1, 1=",  gridinterpolate(1.0_double, 1.0_double, psfGridFitout)

!write(*, *) "Line 755 interpolated value at 0.5, 0.5=",  gridinterpolate(0.5_double, 0.5_double, psfGridFitout)
!write(*, *) "Line 755 interpolated value at -0.5,-0.5=",  gridinterpolate(-0.5_double, -0.5_double, psfGridFitout)

!write(*, *) "Line 755 interpolated value at 0.5, -0.5=",  gridinterpolate(0.5_double, -0.5_double, psfGridFitout)
!write(*, *) "Line 755 interpolated value at 2.0, 2.0=",  gridinterpolate(2.0_double, 2.0_double, psfGridFitout)
!write(*, *) "Line 755 interpolated value at -2.0, -2.0=",  gridinterpolate(-2.0_double, -2.0_double, psfGridFitout)

        end if 

        if(present(bilinearData)) then

            call BilinearConstruct(psf, bilinearData)
        
        end if         

        deallocate(x, y)
        nullify(x, y)
        call CAL_releaseMemory(psf)
        return

    end if ! if binning equals 1





    !******************************************
    ! Finished if binning equals 1
    !*****************************************


    psfnx = nx / 2 + 1
    psfny = ny / 2 + 1
    deallocate(x, y)
        
    x => allocateSingle(1, psfnx, " x")
    y => allocateSingle(1, psfny, " y")


    !*********************************************
    !
    ! Construct the binned image
    !
    !********************************************

    psfImage => allocate2dSingle(0, psfnx - 1, 0, psfny - 1, "psfimage")
    psfImage = 0.0


    do j = 0, psfny - 1
        y1 = dble(j * bin) + 0.5_double
        x(j + 1) = dble(j + 1) - ycentre
      
        do i = 0, psfnx - 1
            x1 = dble(i * bin) + 0.5_double
            y(i + 1) = dble(i + 1) - xcentre
            f1= max(0.0_double, gridinterpolate(x1, y1, psfGridFit1))
            f2= max(0.0_double, gridinterpolate(x1 + 1.0_double, y1, psfGridFit1))
            f3= max(0.0_double, gridinterpolate(x1, y1 + 1.0_double, psfGridFit1))
            f4= max(0.0_double, gridinterpolate(x1 + 1.0_double, y1 + 1.0_double, psfGridFit1))
           
            psfImage(i, j) = f1 + f2 + f3 + f4 
          
        end do
    end do
  
   



    if (present(totalCounts)) then ! Normalize so that psf image contains the required counts
        if (totalCounts .gt. 0.0) then
            f = sum(psfImage)

            psfImage = psfImage * totalCounts  / f

        else
            f = maxval(psfImage)
        
            psfImage = psfImage * abs(totalCounts) / f
            totalCounts = sum(psfImage)

        end if
    else

        f = maxval(psfImage)


        psfimage = psfimage / f



    end if


    xCentre = real(psfnx) / 2.0
    yCentre = real(psfny) / 2.0


    do i = 1, psfnx
        x(i) = real(i) - 0.5 - xCentre
    end do

    do j = 1, psfny

        y(j) = real(j) - 0.5 - yCentre
    end do


    if (present(psfSplineFit)) then    

        call ConstructSpline(psfnx, psfny, x, y, psfImage, psfSplineFit)

        fCentre = interpolateSpline(0.0_double, 0.0_double, psfSplineFit)
!        write(*, *) "Interpolated spline counts at centre=", fCentre

    end if
  
   
    call CAL_releaseMemory(psf)


    if (present(psfGridFitOut)) then ! User wants the grid interpolant
        call GridInterpolantDestruct(psfGridFit1)
  

        call GridInterpolantConstruct(psfnx, psfny, x, y, psfimage, psfGridFit1)

        psfgridfit = psfgridfit1

        deallocate(x, y)

        psfnx = psfnx * 10
        psfny = psfnx


        x => allocateSingle(1, psfnx, "x")
        y => allocateSingle(1, psfny, "y")
        height => allocate2dSingle(0, psfnx - 1, 0,  psfny - 1, "height")


         
        do i = 1, psfnx
            x(i) = real(i) / 10.0 - 0.5 - xCentre
    

            do j = 1, psfny

                y(j) = real(j) / 10.0  - 0.5 - yCentre

!                height(i - 1, j - 1) = max(0.0_double, GridIntegration1(dble(x(i)), dble(y(j))))        
                 height(i - 1, j - 1) = gridinterpolate(dble(x(i)), dble(y(j)), psfGridFit1)

            end do

        end do

       

      
        do i = 1, psfnx
            x(i) = real(i) / 10.0 - 0.5 - xCentre
        end do

        do j = 1, psfny
            y(j) = real(j) / 10.0  - 0.5 - yCentre
        end do



        call GridInterpolantDestruct(psfGridFit1)
         


        call GridInterpolantConstruct(psfnx, psfny, x, y, height, psfGridFitOut)

        psfnx = psfnx / 10
        psfny = psfnx
        deallocate(height)
        nullify(height)


!write(*, *) "Line 933  CreatePSFImage21- gridfit - interpolated value at 0, 0=",  &
!   gridinterpolate(0.0_double, 0.0_double, psfGridFitout)
!write(*, *) "Line 935 interpolated value at -1, -1=",  gridinterpolate(-1.0_double, -1.0_double, psfGridFitout)
!write(*, *) "Line 936 interpolated value at 1, 1=",  gridinterpolate(1.0_double, 1.0_double, psfGridFitout)

!write(*, *) "Line 938 interpolated value at 0.5, 0.5=",  gridinterpolate(0.5_double, 0.5_double, psfGridFitout)
!write(*, *) "Line 939 interpolated value at -0.5,-0.5=",  gridinterpolate(-0.5_double, -0.5_double, psfGridFitout)

!write(*, *) "Line 941 interpolated value at 0.5, -0.5=",  gridinterpolate(0.5_double, -0.5_double, psfGridFitout)
!write(*, *) "Line 942 interpolated value at 2.0, 2.0=",  gridinterpolate(2.0_double, 2.0_double, psfGridFitout)
!write(*, *) "Line 943 interpolated value at -2.0, -2.0=",  gridinterpolate(-2.0_double, -2.0_double, psfGridFitout)





!write(*, *) "line 867"



!    do i = 1, psfnx
!        x(i) = real(i) - 0.5 - xCentre
    

!        do j = 1, psfny

!            y(j) = real(j) - 0.5 - yCentre
!            write(*, *) "x=", x(i), " y=", y(j), &
!               " psfimage(", i - 1, " , ", j - 1, ")=", psfimage(i - 1, j - 1), " grif-interpolated value=", &
!               gridinterpolate(dble(x(i)), dble(y(j)), psfgridfitout)

!        end do

!    end do

!    call GridInterpolantDestruct(psfGridFit1)


    end if  ! Construct grid-interpolant
  


    if(present(bilinearData)) then ! Construct bilinear interpolant

        call BilinearConstruct(psfimage, bilinearData)
           
    end if         



    deallocate(x, y)
    nullify(x, y)


    !*****************************************
    ! May want the actual psf image
    !***************************************** 
    if (present(psfImageOut)) then
        if (associated(psfImageOut)) deallocate(psfimageout)
         
        psfImageOut => psfImage


    else
        deallocate(psfImage)
        nullify(psfImage)
    end if 


end subroutine CreatePSFImage21




end module ompsf_mod






